@@ -6,6 +6,7 @@
 #include "dc_smtp.h"
 #include "dc_saxparser.h"
 #include "dc_job.h"
+#include "dc_oauth2.h"
 
 
 /*******************************************************************************
@@ -737,6 +738,21 @@ void dc_job_do_DC_JOB_CONFIGURE_IMAP(dc_context_t* context, dc_job_t* job)
 	}
 	dc_trim(param->addr);
 
+	if (param->server_flags & DC_LP_AUTH_OAUTH2)
+	{
+		// the used oauth2 addr may differ, check this.
+		// if dc_get_oauth2_addr() is not available in the oauth2 implementation,
+		// just use the given one.
+		PROGRESS(10)
+		char* oauth2_addr = dc_get_oauth2_addr(context, param->addr, param->mail_pw);
+		if (oauth2_addr) {
+			free(param->addr);
+			param->addr = oauth2_addr;
+			dc_sqlite3_set_config(context->sql, "addr", param->addr);
+		}
+		PROGRESS(20)
+	}
+
 	param_domain = strchr(param->addr, '@');
 	if (param_domain==NULL || param_domain[0]==0) {
 		dc_log_error(context, 0, "Bad email-address.");
@@ -765,8 +781,11 @@ void dc_job_do_DC_JOB_CONFIGURE_IMAP(dc_context_t* context, dc_job_t* job)
 	 && param->send_port   ==0
 	 && param->send_user   ==NULL
 	/*&&param->send_pw     ==NULL -- the password cannot be auto-configured and is no criterion for autoconfig or not */
-	 && param->server_flags==0)
+	 && (param->server_flags & (~DC_LP_AUTH_OAUTH2))==0 /* flags but OAuth2 avoid autoconfig */
+	 )
 	{
+		int keep_flags = param->server_flags & DC_LP_AUTH_OAUTH2;
+
 		/* A.  Search configurations from the domain used in the email-address, prefer encrypted */
 		if (param_autoconfig==NULL) {
 			char* url = dc_mprintf("https://autoconfig.%s/mail/config-v1.1.xml?emailaddress=%s", param_domain, param_addr_urlencoded);
@@ -792,7 +811,7 @@ void dc_job_do_DC_JOB_CONFIGURE_IMAP(dc_context_t* context, dc_job_t* job)
 		}
 
 		if (param_autoconfig==NULL) {
-			char* url = dc_mprintf("http://autoconfig.%s/mail/config-v1.1.xml", param_domain); // do not transfer the email-address unencrypted
+			char* url = dc_mprintf("http://autoconfig.%s/mail/config-v1.1.xml?emailaddress=%s", param_domain, param_addr_urlencoded);
 			param_autoconfig = moz_autoconfigure(context, url, param);
 			free(url);
 			PROGRESS(340)
@@ -833,17 +852,8 @@ void dc_job_do_DC_JOB_CONFIGURE_IMAP(dc_context_t* context, dc_job_t* job)
 			/* althoug param_autoconfig's data are no longer needed from, it is important to keep the object as
 			we may enter "deep guessing" if we could not read a configuration */
 		}
-	}
 
-
-	/* 3.  Internal specials (eg. for uploading to chats-folder etc.)
-	 **************************************************************************/
-
-	if (strcasecmp(param_domain, "gmail.com")==0 || strcasecmp(param_domain, "googlemail.com")==0)
-	{
-		/* NB: Checking GMa'l too often (<10 Minutes) may result in blocking, says https://github.com/itprojects/InboxPager/blob/HEAD/README.md#gmail-configuration
-		Also note https://www.google.com/settings/security/lesssecureapps */
-		param->server_flags |= DC_LP_AUTH_XOAUTH2;
+		param->server_flags |= keep_flags;
 	}
 
 
@@ -928,32 +938,49 @@ void dc_job_do_DC_JOB_CONFIGURE_IMAP(dc_context_t* context, dc_job_t* job)
 	PROGRESS(600)
 
 	/* try to connect to IMAP - if we did not got an autoconfig,
-	we do a second try with the localpart of the email-address as the loginname
-	(the part before the '@') */
-	{ char* r = dc_loginparam_get_readable(param); dc_log_info(context, 0, "Trying: %s", r); free(r); }
+	do some further tries with different settings and username variations */
+	for (int username_variation=0; username_variation<=1; username_variation++)
+	{
+		// probe given settings, SSL/993 by default
+		{ char* r = dc_loginparam_get_readable(param); dc_log_info(context, 0, "Trying: %s", r); free(r); }
+		if (dc_imap_connect(context->inbox, param)) {
+			break;
+		}
 
-	if (!dc_imap_connect(context->inbox, param)) {
 		if (param_autoconfig) {
 			goto cleanup;
 		}
 
-		PROGRESS(650)
-
-		char* at = strchr(param->mail_user, '@');
-		if (at) {
-			*at = 0;
-		}
-
-		at = strchr(param->send_user, '@');
-		if (at) {
-			*at = 0;
-		}
+		// probe STARTTLS/993
+		PROGRESS(650+username_variation*30)
+		param->server_flags &= ~DC_LP_IMAP_SOCKET_FLAGS;
+		param->server_flags |=  DC_LP_IMAP_SOCKET_STARTTLS;
 
 		{ char* r = dc_loginparam_get_readable(param); dc_log_info(context, 0, "Trying: %s", r); free(r); }
+		if (dc_imap_connect(context->inbox, param)) {
+			break;
+		}
 
-		if (!dc_imap_connect(context->inbox, param)) {
+		// probe STARTTLS/143
+		PROGRESS(660+username_variation*30)
+		param->mail_port = TYPICAL_IMAP_STARTTLS_PORT;
+		{ char* r = dc_loginparam_get_readable(param); dc_log_info(context, 0, "Trying: %s", r); free(r); }
+		if (dc_imap_connect(context->inbox, param)) {
+			break;
+		}
+		else if (username_variation) {
 			goto cleanup;
 		}
+
+		// next probe round with only the localpart of the email-address as the loginname
+		PROGRESS(670+username_variation*30)
+		param->server_flags &= ~DC_LP_IMAP_SOCKET_FLAGS;
+		param->server_flags |=  DC_LP_IMAP_SOCKET_SSL;
+		param->mail_port    =   TYPICAL_IMAP_SSL_PORT;
+		char* at = strchr(param->mail_user, '@');
+		if (at) { *at = 0; }
+		at = strchr(param->send_user, '@');
+		if (at) { *at = 0; }
 	}
 
 	imap_connected_here = 1;
@@ -974,7 +1001,15 @@ void dc_job_do_DC_JOB_CONFIGURE_IMAP(dc_context_t* context, dc_job_t* job)
 		{ char* r = dc_loginparam_get_readable(param); dc_log_info(context, 0, "Trying: %s", r); free(r); }
 
 		if (!dc_smtp_connect(context->smtp, param)) {
-			goto cleanup;
+			PROGRESS(860)
+
+			param->server_flags &= ~DC_LP_SMTP_SOCKET_FLAGS;
+			param->server_flags |=  DC_LP_SMTP_SOCKET_STARTTLS;
+			param->send_port    =   TYPICAL_SMTP_PLAIN_PORT;
+			{ char* r = dc_loginparam_get_readable(param); dc_log_info(context, 0, "Trying: %s", r); free(r); }
+			if (!dc_smtp_connect(context->smtp, param)) {
+				goto cleanup;
+			}
 		}
 	}
 
