@@ -1,5 +1,6 @@
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <locale.h>
 #include <unistd.h>
 #include <openssl/opensslv.h>
 #include <assert.h>
@@ -36,6 +37,7 @@ static const char* config_keys[] = {
 	,"sentbox_watch"
 	,"mvbox_watch"
 	,"mvbox_move"
+	,"show_emails"
 	,"save_mime_headers"
 	,"configured_addr"
 	,"configured_mail_server"
@@ -165,11 +167,15 @@ static void cb_receive_imf(dc_imap_t* imap, const char* imf_raw_not_terminated,
  *     - If not mentioned otherweise, the callback should return 0.
  * @param userdata can be used by the client for any purpuse.  He finds it
  *     later in dc_get_userdata().
- * @param os_name is only for decorative use and is shown eg. in the `X-Mailer:` header
- *     in the form "Delta Chat <version> for <os_name>".
- *     You can give the name of the operating system and/or the used environment here.
- *     It is okay to give NULL, in this case `X-Mailer:` header is set to "Delta Chat <version>".
- * @return A context object with some public members. The object must be passed to the other context functions
+ * @param os_name is only for decorative use
+ *     and is shown eg. in the `X-Mailer:` header
+ *     in the form "Delta Chat Core <version>/<os_name>".
+ *     You can give the name of the app, the operating system,
+ *     the used environment and/or the version here.
+ *     It is okay to give NULL, in this case `X-Mailer:` header
+ *     is set to "Delta Chat Core <version>".
+ * @return A context object with some public members.
+ *     The object must be passed to the other context functions
  *     and must be freed using dc_context_unref() after usage.
  */
 dc_context_t* dc_context_new(dc_callback_t cb, void* userdata, const char* os_name)
@@ -187,6 +193,7 @@ dc_context_t* dc_context_new(dc_callback_t cb, void* userdata, const char* os_na
 	dc_jobthread_init(&context->mvbox_thread, context, "MVBOX", "configured_mvbox_folder");
 	pthread_mutex_init(&context->smtpidle_condmutex, NULL);
 	pthread_cond_init(&context->smtpidle_cond, NULL);
+	pthread_mutex_init(&context->oauth2_critical, NULL);
 
 	context->magic    = DC_CONTEXT_MAGIC;
 	context->userdata = userdata;
@@ -198,6 +205,8 @@ dc_context_t* dc_context_new(dc_callback_t cb, void* userdata, const char* os_na
 
 	dc_pgp_init();
 	context->sql      = dc_sqlite3_new(context);
+	dc_job_kill_action(context, DC_JOB_EMPTY_SERVER);
+
 	context->inbox    = dc_imap_new(cb_get_config, cb_set_config, cb_precheck_imf, cb_receive_imf, (void*)context, context);
 	context->sentbox_thread.imap = dc_imap_new(cb_get_config, cb_set_config, cb_precheck_imf, cb_receive_imf, (void*)context, context);
 	context->mvbox_thread.imap = dc_imap_new(cb_get_config, cb_set_config, cb_precheck_imf, cb_receive_imf, (void*)context, context);
@@ -258,6 +267,7 @@ void dc_context_unref(dc_context_t* context)
 	dc_jobthread_exit(&context->mvbox_thread);
 	pthread_cond_destroy(&context->smtpidle_cond);
 	pthread_mutex_destroy(&context->smtpidle_condmutex);
+	pthread_mutex_destroy(&context->oauth2_critical);
 
 	free(context->os_name);
 	context->magic = 0;
@@ -505,7 +515,14 @@ static char* get_sys_config_str(const char* key)
  * - `mvbox_move`   = 1=heuristically detect chat-messages
  *                    and move them to the `DeltaChat`-folder,
  *                    0=do not move chat-messages
- * - `save_mime_headers` = 1=save mime headers and make dc_get_mime_headers() work for subsequent calls,
+ * - `show_emails`  = DC_SHOW_EMAILS_OFF (0)=
+ *                    show direct replies to chats only (default),
+ *                    DC_SHOW_EMAILS_ACCEPTED_CONTACTS (1)=
+ *                    also show all mails of confirmed contacts,
+ *                    DC_SHOW_EMAILS_ALL (2)=
+ *                    also show mails of unconfirmed contacts in the deaddrop.
+ * - `save_mime_headers` = 1=save mime headers
+ *                    and make dc_get_mime_headers() work for subsequent calls,
  *                    0=do not save mime headers (default)
  *
  * If you want to retrieve a value, use dc_get_config().
@@ -548,6 +565,14 @@ int dc_set_config(dc_context_t* context, const char* key, const char* value)
 		ret = dc_sqlite3_set_config(context->sql, key, value);
 		dc_interrupt_mvbox_idle(context); // force idle() to be called again with the new mode
 	}
+	else if (strcmp(key, "selfstatus")==0) {
+		// if the status text equals to the default,
+		// store it as NULL to support future updatates of this text
+		char* def = dc_stock_str(context, DC_STR_STATUSLINE);
+		ret = dc_sqlite3_set_config(context->sql, key,
+			(value==NULL || strcmp(value, def)==0)? NULL : value);
+		free(def);
+	}
 	else
 	{
 		ret = dc_sqlite3_set_config(context->sql, key, value);
@@ -627,6 +652,12 @@ char* dc_get_config(dc_context_t* context, const char* key)
 		else if (strcmp(key, "mvbox_move")==0) {
 			value = dc_mprintf("%i", DC_MVBOX_MOVE_DEFAULT);
 		}
+		else if (strcmp(key, "show_emails")==0) {
+			value = dc_mprintf("%i", DC_SHOW_EMAILS_DEFAULT);
+		}
+		else if (strcmp(key, "selfstatus")==0) {
+			value = dc_stock_str(context, DC_STR_STATUSLINE);
+		}
 		else {
 			value = dc_mprintf("");
 		}
@@ -726,11 +757,17 @@ char* dc_get_info(dc_context_t* context)
 	int              deaddrop_msgs = 0;
 	int              is_configured = 0;
 	int              dbversion = 0;
+	int              show_emails = 0;
 	int              mdns_enabled = 0;
 	int              e2ee_enabled = 0;
 	int              prv_key_cnt = 0;
 	int              pub_key_cnt = 0;
 	dc_key_t*        self_public = dc_key_new();
+	int              rpgp_enabled = 0;
+
+	#ifdef DC_USE_RPGP
+		rpgp_enabled = 1;
+	#endif
 
 	dc_strbuilder_t  ret;
 	dc_strbuilder_init(&ret, 0);
@@ -756,6 +793,7 @@ char* dc_get_info(dc_context_t* context)
 	is_configured   = dc_sqlite3_get_config_int(context->sql, "configured", 0);
 	dbversion       = dc_sqlite3_get_config_int(context->sql, "dbversion", 0);
 	e2ee_enabled    = dc_sqlite3_get_config_int(context->sql, "e2ee_enabled", DC_E2EE_DEFAULT_ENABLED);
+	show_emails     = dc_sqlite3_get_config_int(context->sql, "show_emails", DC_SHOW_EMAILS_DEFAULT);
 	mdns_enabled    = dc_sqlite3_get_config_int(context->sql, "mdns_enabled", DC_MDNS_DEFAULT_ENABLED);
 
 	sqlite3_stmt* stmt = dc_sqlite3_prepare(context->sql, "SELECT COUNT(*) FROM keypairs;");
@@ -792,6 +830,7 @@ char* dc_get_info(dc_context_t* context)
 		"sqlite_thread_safe=%i\n"
 		"libetpan_version=%i.%i\n"
 		"openssl_version=%i.%i.%i%c\n"
+		"rpgp_enabled=%i\n"
 		"compile_date=" __DATE__ ", " __TIME__ "\n"
 		"arch=%i\n"
 		"number_of_chats=%i\n"
@@ -812,6 +851,7 @@ char* dc_get_info(dc_context_t* context)
 		"folders_configured=%i\n"
 		"configured_sentbox_folder=%s\n"
 		"configured_mvbox_folder=%s\n"
+		"show_emails=%i\n"
 		"mdns_enabled=%i\n"
 		"e2ee_enabled=%i\n"
 		"private_key_count=%i\n"
@@ -823,6 +863,7 @@ char* dc_get_info(dc_context_t* context)
 		, sqlite3_threadsafe()
 		, libetpan_get_version_major(), libetpan_get_version_minor()
 		, (int)(OPENSSL_VERSION_NUMBER>>28), (int)(OPENSSL_VERSION_NUMBER>>20)&0xFF, (int)(OPENSSL_VERSION_NUMBER>>12)&0xFF, (char)('a'-1+((OPENSSL_VERSION_NUMBER>>4)&0xFF))
+		, rpgp_enabled
 		, sizeof(void*)*8
 		, chats
 		, real_msgs
@@ -842,6 +883,7 @@ char* dc_get_info(dc_context_t* context)
 		, folders_configured
 		, configured_sentbox_folder
 		, configured_mvbox_folder
+		, show_emails
 		, mdns_enabled
 		, e2ee_enabled
 		, prv_key_cnt
@@ -896,6 +938,7 @@ dc_array_t* dc_get_fresh_msgs(dc_context_t* context)
 		" LEFT JOIN contacts ct ON m.from_id=ct.id"
 		" LEFT JOIN chats c ON m.chat_id=c.id"
 		" WHERE m.state=?"
+		"   AND m.hidden=0"
 		"   AND m.chat_id>?"
 		"   AND ct.blocked=0"
 		"   AND (c.blocked=0 OR c.blocked=?)"
