@@ -7,6 +7,7 @@
 #include "dc_imap.h"
 #include "dc_job.h"
 #include "dc_loginparam.h"
+#include "dc_oauth2.h"
 
 
 static int  setup_handle_if_needed   (dc_imap_t*);
@@ -345,7 +346,7 @@ static int fetch_single_msg(dc_imap_t* imap, const char* folder, uint32_t server
 	{
 		struct mailimap_set* set = mailimap_set_new_single(server_uid);
 			r = mailimap_uid_fetch(imap->etpan, set, imap->fetch_type_body, &fetch_result);
-		mailimap_set_free(set);
+		FREE_SET(set);
 	}
 
 	if (dc_imap_is_error(imap, r) || fetch_result==NULL) {
@@ -372,10 +373,7 @@ static int fetch_single_msg(dc_imap_t* imap, const char* folder, uint32_t server
 	imap->receive_imf(imap, msg_content, msg_bytes, folder, server_uid, flags);
 
 cleanup:
-
-	if (fetch_result) {
-		mailimap_fetch_list_free(fetch_result);
-	}
+	FREE_FETCH_LIST(fetch_result);
 	return retry_later? 0 : 1;
 }
 
@@ -390,7 +388,7 @@ static int fetch_from_single_folder(dc_imap_t* imap, const char* folder)
 	size_t               read_cnt = 0;
 	size_t               read_errors = 0;
 	clistiter*           cur;
-	struct mailimap_set* set;
+	struct mailimap_set* set = NULL;
 
 	if (imap==NULL) {
 		goto cleanup;
@@ -419,6 +417,13 @@ static int fetch_from_single_folder(dc_imap_t* imap, const char* folder)
 		if (imap->etpan->imap_selection_info->sel_has_exists) {
 			if (imap->etpan->imap_selection_info->sel_exists <= 0) {
 				dc_log_info(imap->context, 0, "Folder \"%s\" is empty.", folder);
+				if(imap->etpan->imap_selection_info->sel_exists==0) {
+					/* set lastseenuid=0 for empty folders.
+					id we do not do this here, we'll miss the first message
+					as we will get in here again and fetch from lastseenuid+1 then */
+					set_config_lastseenuid(imap, folder,
+						imap->etpan->imap_selection_info->sel_uidvalidity, 0);
+				}
 				goto cleanup;
 			}
 			/* `FETCH <message sequence number> (UID)` */
@@ -431,17 +436,22 @@ static int fetch_from_single_folder(dc_imap_t* imap, const char* folder)
 			set = mailimap_set_new_single(0);
 		}
 		r = mailimap_fetch(imap->etpan, set, imap->fetch_type_prefetch, &fetch_result);
-		mailimap_set_free(set);
+		FREE_SET(set);
 
-		if (dc_imap_is_error(imap, r) || fetch_result==NULL || (cur=clist_begin(fetch_result))==NULL) {
+		if (dc_imap_is_error(imap, r) || fetch_result==NULL) {
+			fetch_result = NULL;
+			dc_log_info(imap->context, 0, "No result returned for folder \"%s\".", folder);
+			goto cleanup; /* this might happen if the mailbox is empty an EXISTS does not work */
+		}
+
+		if ((cur=clist_begin(fetch_result))==NULL) {
 			dc_log_info(imap->context, 0, "Empty result returned for folder \"%s\".", folder);
 			goto cleanup; /* this might happen if the mailbox is empty an EXISTS does not work */
 		}
 
 		struct mailimap_msg_att* msg_att = (struct mailimap_msg_att*)clist_content(cur);
 		lastseenuid = peek_uid(msg_att);
-		mailimap_fetch_list_free(fetch_result);
-		fetch_result = NULL;
+		FREE_FETCH_LIST(fetch_result);
 		if (lastseenuid <= 0) {
 			dc_log_error(imap->context, 0, "Cannot get largest UID for folder \"%s\"", folder);
 			goto cleanup;
@@ -455,12 +465,14 @@ static int fetch_from_single_folder(dc_imap_t* imap, const char* folder)
 		/* store calculated uidvalidity/lastseenuid */
 		uidvalidity = imap->etpan->imap_selection_info->sel_uidvalidity;
 		set_config_lastseenuid(imap, folder, uidvalidity, lastseenuid);
+		dc_log_info(imap->context, 0, "lastseenuid initialized to %i for %s@%i", (int)lastseenuid, folder, (int)uidvalidity);
 	}
 
 	/* fetch messages with larger UID than the last one seen (`UID FETCH lastseenuid+1:*)`, see RFC 4549 */
+	/* CAVE: some servers return UID smaller or equal to the requested ones under some circumstances! */
 	set = mailimap_set_new_interval(lastseenuid+1, 0);
 		r = mailimap_uid_fetch(imap->etpan, set, imap->fetch_type_prefetch, &fetch_result);
-	mailimap_set_free(set);
+	FREE_SET(set);
 
 	if (dc_imap_is_error(imap, r) || fetch_result==NULL)
 	{
@@ -478,19 +490,19 @@ static int fetch_from_single_folder(dc_imap_t* imap, const char* folder)
 	{
 		struct mailimap_msg_att* msg_att = (struct mailimap_msg_att*)clist_content(cur); /* mailimap_msg_att is a list of attributes: list is a list of message attributes */
 		uint32_t cur_uid = peek_uid(msg_att);
-		if (cur_uid > 0
-		 && cur_uid!=lastseenuid /* `UID FETCH <lastseenuid+1>:*` may include lastseenuid if "*"==lastseenuid */)
+		if (cur_uid > lastseenuid /* `UID FETCH <lastseenuid+1>:*` may include lastseenuid if "*"==lastseenuid - and also smaller uids may be returned! */)
 		{
 			char* rfc724_mid = unquote_rfc724_mid(peek_rfc724_mid(msg_att));
 
 			read_cnt++;
 			if (!imap->precheck_imf(imap, rfc724_mid, folder, cur_uid)) {
 				if (fetch_single_msg(imap, folder, cur_uid)==0/* 0=try again later*/) {
+					dc_log_info(imap->context, 0, "Read error for message %s from \"%s\", trying over later.", rfc724_mid, folder);
 					read_errors++; // with read_errors, lastseenuid is not written
 				}
 			}
 			else {
-				dc_log_info(imap->context, 0, "Skipping message %s by precheck.", rfc724_mid);
+				dc_log_info(imap->context, 0, "Skipping message %s from \"%s\" by precheck.", rfc724_mid, folder);
 			}
 
 			if (cur_uid > new_lastseenuid) {
@@ -502,6 +514,8 @@ static int fetch_from_single_folder(dc_imap_t* imap, const char* folder)
 	}
 
 	if (!read_errors && new_lastseenuid > 0) {
+		// TODO: it might be better to increase the lastseenuid also on partial errors.
+		// however, this requires to sort the list before going through it above.
 		set_config_lastseenuid(imap, folder, uidvalidity, new_lastseenuid);
 	}
 
@@ -515,10 +529,7 @@ cleanup:
 		dc_log_info(imap->context, 0, "%i mails read from \"%s\".", (int)read_cnt, folder);
 	}
 
-	if (fetch_result) {
-		mailimap_fetch_list_free(fetch_result);
-	}
-
+	FREE_FETCH_LIST(fetch_result);
 	return read_cnt;
 }
 
@@ -759,19 +770,21 @@ static int setup_handle_if_needed(dc_imap_t* imap)
 		dc_log_info(imap->context, 0, "IMAP-server %s:%i SSL-connected.", imap->imap_server, (int)imap->imap_port);
 	}
 
-	/* TODO: There are more authorisation types, see mailcore2/MCIMAPSession.cpp, however, I'm not sure of they are really all needed */
-	/*if (imap->server_flags&DC_LP_AUTH_XOAUTH2)
+	/* from mailcore2/MCIMAPSession.cpp */
+	if (imap->server_flags&DC_LP_AUTH_OAUTH2)
 	{
-		//TODO: Support XOAUTH2, we "just" need to get the token someway. If we do so, there is no more need for the user to enable
-		//https://www.google.com/settings/security/lesssecureapps - however, maybe this is also not needed if the user had enabled 2-factor-authorisation.
-		if (mOAuth2Token==NULL) {
-			r = MAILIMAP_ERROR_STREAM;
-		}
-		else {
-			r = mailimap_oauth2_authenticate(imap->etpan, imap->imap_use, mOAuth2Token);
+		// for DC_LP_AUTH_OAUTH2, user_pw is assumed to be the oauth_token
+		dc_log_info(imap->context, 0, "IMAP-OAuth2 connect...");
+		char* access_token = dc_get_oauth2_access_token(imap->context, imap->addr, imap->imap_pw, 0);
+		r = mailimap_oauth2_authenticate(imap->etpan, imap->imap_user, access_token);
+		if (dc_imap_is_error(imap, r)) {
+			free(access_token);
+			access_token = dc_get_oauth2_access_token(imap->context, imap->addr, imap->imap_pw, DC_REGENERATE);
+			r = mailimap_oauth2_authenticate(imap->etpan, imap->imap_user, access_token);
 		}
+		free(access_token);
 	}
-	else*/
+	else
 	{
 		/* DC_LP_AUTH_NORMAL or no auth flag set */
 		r = mailimap_login(imap->etpan, imap->imap_user, imap->imap_pw);
@@ -837,6 +850,9 @@ static void unsetup_handle(dc_imap_t* imap)
 
 static void free_connect_param(dc_imap_t* imap)
 {
+	free(imap->addr);
+	imap->addr = NULL;
+
 	free(imap->imap_server);
 	imap->imap_server = NULL;
 
@@ -870,6 +886,7 @@ int dc_imap_connect(dc_imap_t* imap, const dc_loginparam_t* lp)
 		goto cleanup;
 	}
 
+	imap->addr         = dc_strdup(lp->addr);
 	imap->imap_server  = dc_strdup(lp->mail_server);
 	imap->imap_port    = lp->mail_port;
 	imap->imap_user    = dc_strdup(lp->mail_user);
@@ -1057,9 +1074,7 @@ cleanup:
 	if (store_att_flags) {
 		mailimap_store_att_flags_free(store_att_flags);
 	}
-	if (set) {
-		mailimap_set_free(set);
-	}
+	FREE_SET(set);
 	return imap->should_reconnect? 0 : 1; /* all non-connection states are treated as success - the mail may already be deleted or moved away on the server */
 }
 
@@ -1224,6 +1239,7 @@ dc_imap_res dc_imap_set_mdnsent(dc_imap_t* imap, const char* folder, uint32_t ui
 	{
 		int r = mailimap_uid_fetch(imap->etpan, set, imap->fetch_type_flags, &fetch_result);
 		if (dc_imap_is_error(imap, r) || fetch_result==NULL) {
+			fetch_result = NULL;
 			goto cleanup;
 		}
 
@@ -1266,7 +1282,7 @@ int dc_imap_delete_msg(dc_imap_t* imap, const char* rfc724_mid, const char* fold
 	char*  is_rfc724_mid = NULL;
 	char*  new_folder = NULL;
 
-	if (imap==NULL || rfc724_mid==NULL || folder==NULL || folder[0]==0) {
+	if (imap==NULL || rfc724_mid==NULL || folder==NULL || folder[0]==0 || server_uid==0) {
 		success = 1; /* job done, do not try over */
 		goto cleanup;
 	}
@@ -1281,34 +1297,30 @@ int dc_imap_delete_msg(dc_imap_t* imap, const char* rfc724_mid, const char* fold
 	/* check if Folder+UID matches the Message-ID (to detect if the messages
 	was moved around by other MUAs and in place of an UIDVALIDITY check)
 	*/
-	if (server_uid)
 	{
 		clistiter* cur = NULL;
 		const char* is_quoted_rfc724_mid = NULL;
 
 		struct mailimap_set* set = mailimap_set_new_single(server_uid);
 			r = mailimap_uid_fetch(imap->etpan, set, imap->fetch_type_prefetch, &fetch_result);
-		mailimap_set_free(set);
+		FREE_SET(set);
 
-		if (dc_imap_is_error(imap, r) || fetch_result==NULL
-		 || (cur=clist_begin(fetch_result))==NULL
+		if (dc_imap_is_error(imap, r) || fetch_result==NULL) {
+			fetch_result = NULL;
+			dc_log_warning(imap->context, 0, "Cannot delete on IMAP, %s/%i not found.", folder, (int)server_uid);
+			server_uid = 0;
+		}
+
+		if( (cur=clist_begin(fetch_result))==NULL
 		 || (is_quoted_rfc724_mid=peek_rfc724_mid((struct mailimap_msg_att*)clist_content(cur)))==NULL
 		 || (is_rfc724_mid=unquote_rfc724_mid(is_quoted_rfc724_mid))==NULL
 		 || strcmp(is_rfc724_mid, rfc724_mid)!=0)
 		{
-			dc_log_warning(imap->context, 0, "UID not found in the given folder or does not match Message-ID.");
+			dc_log_warning(imap->context, 0, "Cannot delete on IMAP, %s/%i does not match %s.", folder, (int)server_uid, rfc724_mid);
 			server_uid = 0;
 		}
 	}
 
-	/* server_uid is 0 now if it was not given or if it does not match the given message id;
-	try to search for it in all folders (the message may be moved by another MUA to a folder we do not sync or the sync is a moment ago) */
-	if (server_uid==0) {
-			dc_log_warning(imap->context, 0, "Message-ID \"%s\" not found in any folder, cannot delete message.", rfc724_mid);
-			goto cleanup;
-	}
-
-
 	/* mark the message for deletion */
 	if (add_flag(imap, server_uid, mailimap_flag_new_deleted())==0) {
 		dc_log_warning(imap->context, 0, "Cannot mark message as \"Deleted\"."); /* maybe the message is already deleted */
@@ -1322,7 +1334,7 @@ int dc_imap_delete_msg(dc_imap_t* imap, const char* rfc724_mid, const char* fold
 
 cleanup:
 
-	if (fetch_result) { mailimap_fetch_list_free(fetch_result); }
+	FREE_FETCH_LIST(fetch_result);
 	free(is_rfc724_mid);
 	free(new_folder);
 
@@ -1330,3 +1342,40 @@ cleanup:
 
 }
 
+
+void dc_imap_empty_folder(dc_imap_t* imap, const char* folder)
+{
+	struct mailimap_flag_list*       flag_list = NULL;
+	struct mailimap_store_att_flags* store_att_flags = NULL;
+	struct mailimap_set*             set = NULL;
+
+	if (imap==NULL || folder==NULL || folder[0]==0) {
+		goto cleanup;
+	}
+
+	dc_log_info(imap->context, 0, "Emptying folder \"%s\" ...", folder);
+
+	if (select_folder(imap, folder)==0) {
+		dc_log_warning(imap->context, 0, "Cannot select folder %s for emptying.", folder);
+		goto cleanup;
+	}
+
+	set = mailimap_set_new_interval(1/*start with smallest uid*/, 0/*0=`*`=largets uid*/);
+
+	flag_list = mailimap_flag_list_new_empty();
+	mailimap_flag_list_add(flag_list, mailimap_flag_new_deleted());
+	store_att_flags = mailimap_store_att_flags_new_add_flags(flag_list); /* FLAGS.SILENT does not return the new value */
+
+	mailimap_uid_store(imap->etpan, set, store_att_flags);
+
+	imap->selected_folder_needs_expunge = 1;
+	select_folder(imap, NULL);
+
+	dc_log_info(imap->context, 0, "Emptying folder \"%s\" done.", folder);
+
+cleanup:
+	if (store_att_flags) {
+		mailimap_store_att_flags_free(store_att_flags);
+	}
+	FREE_SET(set);
+}
