@@ -5,6 +5,8 @@
 #include "dc_simplify.h"
 
 
+static void hash_header(dc_hash_t* out, const struct mailimf_fields* in, dc_context_t* context);
+
 
 // deprecated: flag to switch generation of compound messages on and off.
 static int s_generate_compound_msgs = 1;
@@ -938,6 +940,41 @@ void dc_mimeparser_empty(dc_mimeparser_t* mimeparser)
 	mimeparser->decrypting_failed = 0;
 
 	dc_e2ee_thanks(mimeparser->e2ee_helper);
+
+	dc_kml_unref(mimeparser->location_kml);
+	mimeparser->location_kml = NULL;
+
+	dc_kml_unref(mimeparser->message_kml);
+	mimeparser->message_kml = NULL;
+}
+
+
+void dc_mimeparser_repl_msg_by_error(dc_mimeparser_t* mimeparser,
+                                     const char* error_msg)
+{
+	dc_mimepart_t* part = NULL;
+	int            i = 0;
+
+	if (mimeparser==NULL || mimeparser->parts==NULL
+	 || carray_count(mimeparser->parts)<=0) {
+		return;
+	}
+
+	// part->raw_msg is unchanged
+	// so that the original message can be retrieved using dc_get_msg_info()
+	part = (dc_mimepart_t*)carray_get(mimeparser->parts, 0);
+	part->type = DC_MSG_TEXT;
+	free(part->msg);
+	part->msg = dc_mprintf(DC_EDITORIAL_OPEN "%s" DC_EDITORIAL_CLOSE, error_msg);
+
+	for (i = 1; i < carray_count(mimeparser->parts); i++) {
+		part = (dc_mimepart_t*)carray_get(mimeparser->parts, i);
+		if (part) {
+			dc_mimepart_unref(part);
+		}
+	}
+
+	carray_set_size(mimeparser->parts, 1);
 }
 
 
@@ -1064,7 +1101,13 @@ static int dc_mimeparser_add_single_part_if_known(dc_mimeparser_t* mimeparser, s
 					}
 				}
 
-				char* simplified_txt = dc_simplify_simplify(simplifier, decoded_data, decoded_data_bytes, mime_type==DC_MIMETYPE_TEXT_HTML? 1 : 0);
+				/* check header directly as is_send_by_messenger is not yet set up */
+				int is_msgrmsg = dc_mimeparser_lookup_optional_field(mimeparser, "Chat-Version")!=NULL;
+
+				char* simplified_txt = dc_simplify_simplify(simplifier,
+					decoded_data, decoded_data_bytes,
+					mime_type==DC_MIMETYPE_TEXT_HTML? 1 : 0,
+					is_msgrmsg);
 				if (simplified_txt && simplified_txt[0])
 				{
 					part = dc_mimepart_new();
@@ -1154,6 +1197,20 @@ static int dc_mimeparser_add_single_part_if_known(dc_mimeparser_t* mimeparser, s
 					}
 				}
 
+				if (strncmp(desired_filename, "location", 8)==0
+				 && strncmp(desired_filename+strlen(desired_filename)-4, ".kml", 4)==0) {
+					mimeparser->location_kml = dc_kml_parse(mimeparser->context,
+						decoded_data, decoded_data_bytes);
+					goto cleanup;
+				}
+
+				if (strncmp(desired_filename, "message", 7)==0
+				 && strncmp(desired_filename+strlen(desired_filename)-4, ".kml", 4)==0) {
+					mimeparser->message_kml = dc_kml_parse(mimeparser->context,
+						decoded_data, decoded_data_bytes);
+					goto cleanup;
+				}
+
 				dc_replace_bad_utf8_chars(desired_filename);
 
 				do_add_single_file_part(mimeparser, msg_type, mime_type, raw_mime, decoded_data, decoded_data_bytes, desired_filename);
@@ -1204,6 +1261,9 @@ static int dc_mimeparser_parse_mime_recursive(dc_mimeparser_t* mimeparser, struc
 			 || mimeparser->header_protected==NULL) {
 				dc_log_warning(mimeparser->context, 0, "Protected headers parsing error.");
 			}
+			else {
+				hash_header(&mimeparser->header, mimeparser->header_protected, mimeparser->context);
+			}
 		}
 		else {
 			dc_log_info(mimeparser->context, 0, "Protected headers found in MIME header: Will be ignored as we already found an outer one.");
@@ -1348,6 +1408,7 @@ static int dc_mimeparser_parse_mime_recursive(dc_mimeparser_t* mimeparser, struc
 			if (mimeparser->header_root==NULL)
 			{
 				mimeparser->header_root = mime->mm_data.mm_message.mm_fields;
+				hash_header(&mimeparser->header, mimeparser->header_root, mimeparser->context);
 			}
 
 			if (mime->mm_data.mm_message.mm_msg_mime)
@@ -1437,8 +1498,9 @@ static void hash_header(dc_hash_t* out, const struct mailimf_fields* in, dc_cont
  */
 void dc_mimeparser_parse(dc_mimeparser_t* mimeparser, const char* body_not_terminated, size_t body_bytes)
 {
-	int    r = 0;
-	size_t index = 0;
+	int                            r = 0;
+	size_t                         index = 0;
+	struct mailimf_optional_field* optional_field = NULL;
 
 	dc_mimeparser_empty(mimeparser);
 
@@ -1459,10 +1521,6 @@ void dc_mimeparser_parse(dc_mimeparser_t* mimeparser, const char* body_not_termi
 	/* recursively check, whats parsed, this also sets up header_old */
 	dc_mimeparser_parse_mime_recursive(mimeparser, mimeparser->mimeroot);
 
-	/* setup header */
-	hash_header(&mimeparser->header, mimeparser->header_root, mimeparser->context);
-	hash_header(&mimeparser->header, mimeparser->header_protected, mimeparser->context); /* overwrite the original header with the protected one */
-
 	/* set some basic data */
 	{
 		struct mailimf_field* field = dc_mimeparser_lookup_field(mimeparser, "Subject");
@@ -1496,7 +1554,13 @@ void dc_mimeparser_parse(dc_mimeparser_t* mimeparser, const char* body_not_termi
 				}
 			}
 		}
-		mimeparser->is_send_by_messenger = 0; /* do not treat a setup message as a messenger message (eg. do not move setup messages to the Chats-folder; there may be a 3rd device that wants to handle it) */
+	}
+	else if ((optional_field=dc_mimeparser_lookup_optional_field(mimeparser, "Chat-Content"))!=NULL
+	       && optional_field->fld_value!=NULL)
+	{
+		if (strcmp(optional_field->fld_value, "location-streaming-enabled")==0) {
+			mimeparser->is_system_message = DC_CMD_LOCATION_STREAMING_ENABLED;
+		}
 	}
 
 	/* some special system message? */
