@@ -32,12 +32,20 @@ safe. However, there are some points to keep in mind:
 void dc_sqlite3_log_error(dc_sqlite3_t* sql, const char* msg_format, ...)
 {
 	char*       msg = NULL;
-	const char* notSetUp = "SQLite object not set up.";
 	va_list     va;
+
+	if (sql==NULL || msg_format==NULL) {
+		return;
+	}
+
 	va_start(va, msg_format);
-		msg = sqlite3_vmprintf(msg_format, va); if (msg==NULL) { dc_log_error(sql->context, 0, "Bad log format string \"%s\".", msg_format); }
-			dc_log_error(sql->context, 0, "%s SQLite says: %s", msg, sql->cobj? sqlite3_errmsg(sql->cobj) : notSetUp);
-		sqlite3_free(msg);
+	msg = sqlite3_vmprintf(msg_format, va);
+
+	dc_log_error(sql->context, 0, "%s SQLite says: %s",
+		msg? msg : "",
+		sql->cobj? sqlite3_errmsg(sql->cobj) : "SQLite object not set up.");
+
+	sqlite3_free(msg);
 	va_end(va);
 }
 
@@ -64,6 +72,33 @@ sqlite3_stmt* dc_sqlite3_prepare(dc_sqlite3_t* sql, const char* querystr)
 }
 
 
+int dc_sqlite3_try_execute(dc_sqlite3_t* sql, const char* querystr)
+{
+	// same as dc_sqlite3_execute() but does not pass error to ui
+	int           success = 0;
+	sqlite3_stmt* stmt = NULL;
+	int           sql_state = 0;
+
+	stmt = dc_sqlite3_prepare(sql, querystr);
+	if (stmt==NULL) {
+		goto cleanup;
+	}
+
+	sql_state = sqlite3_step(stmt);
+	if (sql_state != SQLITE_DONE && sql_state != SQLITE_ROW)  {
+		dc_log_warning(sql->context, 0, "Try-execute for \"%s\" failed: %s",
+			querystr, sqlite3_errmsg(sql->cobj));
+		goto cleanup;
+	}
+
+	success = 1;
+
+cleanup:
+	sqlite3_finalize(stmt);
+	return success;
+}
+
+
 int dc_sqlite3_execute(dc_sqlite3_t* sql, const char* querystr)
 {
 	int           success = 0;
@@ -77,16 +112,14 @@ int dc_sqlite3_execute(dc_sqlite3_t* sql, const char* querystr)
 
 	sqlState = sqlite3_step(stmt);
 	if (sqlState != SQLITE_DONE && sqlState != SQLITE_ROW)  {
-		dc_sqlite3_log_error(sql, "Cannot excecute \"%s\".", querystr);
+		dc_sqlite3_log_error(sql, "Cannot execute \"%s\".", querystr);
 		goto cleanup;
 	}
 
 	success = 1;
 
 cleanup:
-	if (stmt) {
-		sqlite3_finalize(stmt);
-	}
+	sqlite3_finalize(stmt);
 	return success;
 }
 
@@ -108,6 +141,27 @@ uint32_t dc_sqlite3_get_rowid(dc_sqlite3_t* sql, const char* table, const char*
 }
 
 
+uint32_t dc_sqlite3_get_rowid2(dc_sqlite3_t* sql, const char* table,
+                               const char* field, uint64_t value,
+                               const char* field2, uint32_t value2)
+{
+	// same as dc_sqlite3_get_rowid() with a key over two columns
+	uint32_t id = 0;
+
+	// see https://www.sqlite.org/printf.html for sqlite-printf modifiers
+	char* q3 = sqlite3_mprintf(
+		"SELECT id FROM %s WHERE %s=%lli AND %s=%i ORDER BY id DESC;",
+		table, field, value, field2, value2);
+	sqlite3_stmt* stmt = dc_sqlite3_prepare(sql, q3);
+	if (SQLITE_ROW==sqlite3_step(stmt)) {
+		id = sqlite3_column_int(stmt, 0);
+	}
+	sqlite3_finalize(stmt);
+	sqlite3_free(q3);
+	return id;
+}
+
+
 dc_sqlite3_t* dc_sqlite3_new(dc_context_t* context)
 {
 	dc_sqlite3_t* sql = NULL;
@@ -184,6 +238,7 @@ int dc_sqlite3_open(dc_sqlite3_t* sql, const char* dbfile, int flags)
 
 	if (!(flags&DC_OPEN_READONLY))
 	{
+		int exists_before_update = 0;
 		int dbversion_before_update = 0;
 
 		/* Init tables to dbversion=0 */
@@ -198,7 +253,7 @@ int dc_sqlite3_open(dc_sqlite3_t* sql, const char* dbfile, int flags)
 			dc_sqlite3_execute(sql, "CREATE TABLE config (id INTEGER PRIMARY KEY, keyname TEXT, value TEXT);");
 			dc_sqlite3_execute(sql, "CREATE INDEX config_index1 ON config (keyname);");
 
-			dc_sqlite3_execute(sql, "CREATE TABLE contacts (id INTEGER PRIMARY KEY,"
+			dc_sqlite3_execute(sql, "CREATE TABLE contacts (id INTEGER PRIMARY KEY AUTOINCREMENT,"
 						" name TEXT DEFAULT '',"
 						" addr TEXT DEFAULT '' COLLATE NOCASE,"
 						" origin INTEGER DEFAULT 0,"
@@ -212,7 +267,7 @@ int dc_sqlite3_open(dc_sqlite3_t* sql, const char* dbfile, int flags)
 				#error
 			#endif
 
-			dc_sqlite3_execute(sql, "CREATE TABLE chats (id INTEGER PRIMARY KEY, "
+			dc_sqlite3_execute(sql, "CREATE TABLE chats (id INTEGER PRIMARY KEY AUTOINCREMENT, "
 						" type INTEGER DEFAULT 0,"
 						" name TEXT DEFAULT '',"
 						" draft_timestamp INTEGER DEFAULT 0,"
@@ -231,7 +286,7 @@ int dc_sqlite3_open(dc_sqlite3_t* sql, const char* dbfile, int flags)
 				#error
 			#endif
 
-			dc_sqlite3_execute(sql, "CREATE TABLE msgs (id INTEGER PRIMARY KEY,"
+			dc_sqlite3_execute(sql, "CREATE TABLE msgs (id INTEGER PRIMARY KEY AUTOINCREMENT,"
 						" rfc724_mid TEXT DEFAULT '',"     /* forever-global-unique Message-ID-string, unfortunately, this cannot be easily used to communicate via IMAP */
 						" server_folder TEXT DEFAULT '',"  /* folder as used on the server, the folder will change when messages are moved around. */
 						" server_uid INTEGER DEFAULT 0,"   /* UID as used on the server, the UID will change when messages are moved around, unique together with validity, see RFC 3501; the validity may differ from folder to folder.  We use the server_uid for "markseen" and to delete messages as we check against the message-id, we ignore the validity for these commands. */
@@ -252,7 +307,7 @@ int dc_sqlite3_open(dc_sqlite3_t* sql, const char* dbfile, int flags)
 			dc_sqlite3_execute(sql, "CREATE INDEX msgs_index4 ON msgs (state);");          /* for selecting the count of fresh messages (as there are normally only few unread messages, an index over the chat_id is not required for _this_ purpose */
 			dc_sqlite3_execute(sql, "INSERT INTO msgs (id,msgrmsg,txt) VALUES (1,0,'marker1'), (2,0,'rsvd'), (3,0,'rsvd'), (4,0,'rsvd'), (5,0,'rsvd'), (6,0,'rsvd'), (7,0,'rsvd'), (8,0,'rsvd'), (9,0,'daymarker');"); /* make sure, the reserved IDs are not used */
 
-			dc_sqlite3_execute(sql, "CREATE TABLE jobs (id INTEGER PRIMARY KEY,"
+			dc_sqlite3_execute(sql, "CREATE TABLE jobs (id INTEGER PRIMARY KEY AUTOINCREMENT,"
 						" added_timestamp INTEGER,"
 						" desired_timestamp INTEGER DEFAULT 0,"
 						" action INTEGER,"
@@ -272,6 +327,7 @@ int dc_sqlite3_open(dc_sqlite3_t* sql, const char* dbfile, int flags)
 		}
 		else
 		{
+			exists_before_update = 1;
 			dbversion_before_update = dc_sqlite3_get_config_int(sql, "dbversion", 0);
 		}
 
@@ -510,6 +566,77 @@ int dc_sqlite3_open(dc_sqlite3_t* sql, const char* dbfile, int flags)
 			}
 		#undef NEW_DB_VERSION
 
+		#define NEW_DB_VERSION 49
+			if (dbversion < NEW_DB_VERSION)
+			{
+				dc_sqlite3_execute(sql, "ALTER TABLE chats ADD COLUMN gossiped_timestamp INTEGER DEFAULT 0;");
+
+				dbversion = NEW_DB_VERSION;
+				dc_sqlite3_set_config_int(sql, "dbversion", NEW_DB_VERSION);
+			}
+		#undef NEW_DB_VERSION
+
+		#define NEW_DB_VERSION 50
+			if (dbversion < NEW_DB_VERSION)
+			{
+				/* installations <= 0.100.1 used DC_SHOW_EMAILS_ALL implicitly;
+				keep this default and use DC_SHOW_EMAILS_NO
+				only for new installations */
+				if (exists_before_update) {
+					dc_sqlite3_set_config_int(sql, "show_emails", DC_SHOW_EMAILS_ALL);
+				}
+
+				dbversion = NEW_DB_VERSION;
+				dc_sqlite3_set_config_int(sql, "dbversion", NEW_DB_VERSION);
+			}
+		#undef NEW_DB_VERSION
+
+		#define NEW_DB_VERSION 53
+			if (dbversion < NEW_DB_VERSION)
+			{
+				// the messages containing _only_ locations
+				// are also added to the database as _hidden_.
+				dc_sqlite3_execute(sql, "CREATE TABLE locations ("
+							" id INTEGER PRIMARY KEY AUTOINCREMENT,"
+							" latitude REAL DEFAULT 0.0,"
+							" longitude REAL DEFAULT 0.0,"
+							" accuracy REAL DEFAULT 0.0,"
+							" timestamp INTEGER DEFAULT 0,"
+							" chat_id INTEGER DEFAULT 0,"
+							" from_id INTEGER DEFAULT 0);");
+				dc_sqlite3_execute(sql, "CREATE INDEX locations_index1 ON locations (from_id);");
+				dc_sqlite3_execute(sql, "CREATE INDEX locations_index2 ON locations (timestamp);");
+				dc_sqlite3_execute(sql, "ALTER TABLE chats ADD COLUMN locations_send_begin INTEGER DEFAULT 0;");
+				dc_sqlite3_execute(sql, "ALTER TABLE chats ADD COLUMN locations_send_until INTEGER DEFAULT 0;");
+				dc_sqlite3_execute(sql, "ALTER TABLE chats ADD COLUMN locations_last_sent INTEGER DEFAULT 0;");
+				dc_sqlite3_execute(sql, "CREATE INDEX chats_index3 ON chats (locations_send_until);");
+
+				dbversion = NEW_DB_VERSION;
+				dc_sqlite3_set_config_int(sql, "dbversion", NEW_DB_VERSION);
+			}
+		#undef NEW_DB_VERSION
+
+		#define NEW_DB_VERSION 54
+			if (dbversion < NEW_DB_VERSION)
+			{
+				dc_sqlite3_execute(sql, "ALTER TABLE msgs ADD COLUMN location_id INTEGER DEFAULT 0;");
+				dc_sqlite3_execute(sql, "CREATE INDEX msgs_index6 ON msgs (location_id);");
+
+				dbversion = NEW_DB_VERSION;
+				dc_sqlite3_set_config_int(sql, "dbversion", NEW_DB_VERSION);
+			}
+		#undef NEW_DB_VERSION
+
+		#define NEW_DB_VERSION 55
+			if (dbversion < NEW_DB_VERSION)
+			{
+				dc_sqlite3_execute(sql, "ALTER TABLE locations ADD COLUMN independent INTEGER DEFAULT 0;");
+
+				dbversion = NEW_DB_VERSION;
+				dc_sqlite3_set_config_int(sql, "dbversion", NEW_DB_VERSION);
+			}
+		#undef NEW_DB_VERSION
+
 		// (2) updates that require high-level objects
 		// (the structure is complete now and all objects are usable)
 		// --------------------------------------------------------------------
@@ -729,6 +856,19 @@ int32_t dc_sqlite3_get_config_int(dc_sqlite3_t* sql, const char* key, int32_t de
 }
 
 
+int64_t dc_sqlite3_get_config_int64(dc_sqlite3_t* sql, const char* key, int64_t def)
+{
+    char* str = dc_sqlite3_get_config(sql, key, NULL);
+    if (str==NULL) {
+		return def;
+    }
+    int64_t ret = 0;
+    sscanf(str, "%"SCNd64, &ret);
+    free(str);
+    return ret;
+}
+
+
 int dc_sqlite3_set_config_int(dc_sqlite3_t* sql, const char* key, int32_t value)
 {
     char* value_str = dc_mprintf("%i", (int)value);
@@ -741,6 +881,18 @@ int dc_sqlite3_set_config_int(dc_sqlite3_t* sql, const char* key, int32_t value)
 }
 
 
+int dc_sqlite3_set_config_int64(dc_sqlite3_t* sql, const char* key, int64_t value)
+{
+    char* value_str = dc_mprintf("%"PRId64, (long)value);
+    if (value_str==NULL) {
+		return 0;
+    }
+    int ret = dc_sqlite3_set_config(sql, key, value_str);
+    free(value_str);
+    return ret;
+}
+
+
 /*******************************************************************************
  * Transactions
  ******************************************************************************/
@@ -866,6 +1018,10 @@ void dc_housekeeping(dc_context_t* context)
 		"   AND type!=" DC_STRINGIFY(DC_MSG_TEXT) ";",
 		DC_PARAM_FILE);
 
+	maybe_add_from_param(context, &files_in_use,
+		"SELECT param FROM jobs;",
+		DC_PARAM_FILE);
+
 	maybe_add_from_param(context, &files_in_use,
 		"SELECT param FROM chats;",
 		DC_PARAM_PROFILE_IMAGE);
