@@ -11,7 +11,7 @@ extern "C" {
 #endif
 
 
-#define DC_VERSION_STR "0.39.0"
+#define DC_VERSION_STR "0.45.0"
 
 
 /**
@@ -227,6 +227,7 @@ char*           dc_get_blobdir               (const dc_context_t*);
 int             dc_set_config                (dc_context_t*, const char* key, const char* value);
 char*           dc_get_config                (dc_context_t*, const char* key);
 char*           dc_get_info                  (dc_context_t*);
+char*           dc_get_oauth2_url            (dc_context_t*, const char* addr, const char* redirect);
 char*           dc_get_version_str           (void);
 void            dc_openssl_init_not_required (void);
 void            dc_no_compound_msgs          (void); // deprecated
@@ -268,6 +269,7 @@ uint32_t        dc_create_chat_by_msg_id     (dc_context_t*, uint32_t msg_id);
 uint32_t        dc_create_chat_by_contact_id (dc_context_t*, uint32_t contact_id);
 uint32_t        dc_get_chat_id_by_contact_id (dc_context_t*, uint32_t contact_id);
 
+uint32_t        dc_prepare_msg               (dc_context_t*, uint32_t chat_id, dc_msg_t*);
 uint32_t        dc_send_msg                  (dc_context_t*, uint32_t chat_id, dc_msg_t*);
 uint32_t        dc_send_text_msg             (dc_context_t*, uint32_t chat_id, const char* text_to_send);
 void            dc_set_draft                 (dc_context_t*, uint32_t chat_id, dc_msg_t*);
@@ -341,6 +343,9 @@ int             dc_check_password            (dc_context_t*, const char* pw);
 char*           dc_initiate_key_transfer     (dc_context_t*);
 int             dc_continue_key_transfer     (dc_context_t*, uint32_t msg_id, const char* setup_code);
 void            dc_stop_ongoing_process      (dc_context_t*);
+#define         DC_EMPTY_MVBOX               0x01
+#define         DC_EMPTY_INBOX               0x02
+void            dc_empty_server              (dc_context_t*, int flags);
 
 
 // out-of-band verification
@@ -358,6 +363,14 @@ char*           dc_get_securejoin_qr         (dc_context_t*, uint32_t chat_id);
 uint32_t        dc_join_securejoin           (dc_context_t*, const char* qr);
 
 
+// location streaming
+void        dc_send_locations_to_chat       (dc_context_t*, uint32_t chat_id, int seconds);
+int         dc_is_sending_locations_to_chat (dc_context_t*, uint32_t chat_id);
+int         dc_set_location                 (dc_context_t*, double latitude, double longitude, double accuracy);
+dc_array_t* dc_get_locations                (dc_context_t*, uint32_t chat_id, uint32_t contact_id, time_t timestamp_begin, time_t timestamp_end);
+void        dc_delete_all_locations         (dc_context_t*);
+
+
 /**
  * @class dc_array_t
  *
@@ -366,8 +379,6 @@ uint32_t        dc_join_securejoin           (dc_context_t*, const char* qr);
  * The items of the array are typically IDs.
  * To free an array object, use dc_array_unref().
  */
-dc_array_t*      dc_array_new                (dc_context_t*, size_t initsize);
-void             dc_array_empty              (dc_array_t*);
 void             dc_array_unref              (dc_array_t*);
 
 void             dc_array_add_uint           (dc_array_t*, uintptr_t);
@@ -378,6 +389,15 @@ size_t           dc_array_get_cnt            (const dc_array_t*);
 uintptr_t        dc_array_get_uint           (const dc_array_t*, size_t index);
 uint32_t         dc_array_get_id             (const dc_array_t*, size_t index);
 void*            dc_array_get_ptr            (const dc_array_t*, size_t index);
+double           dc_array_get_latitude       (const dc_array_t*, size_t index);
+double           dc_array_get_longitude      (const dc_array_t*, size_t index);
+double           dc_array_get_accuracy       (const dc_array_t*, size_t index);
+time_t           dc_array_get_timestamp      (const dc_array_t*, size_t index);
+uint32_t         dc_array_get_chat_id        (const dc_array_t*, size_t index);
+uint32_t         dc_array_get_contact_id     (const dc_array_t*, size_t index);
+uint32_t         dc_array_get_msg_id         (const dc_array_t*, size_t index);
+char*            dc_array_get_marker         (const dc_array_t*, size_t index);
+int              dc_array_is_independent     (const dc_array_t*, size_t index);
 
 int              dc_array_search_id          (const dc_array_t*, uint32_t needle, size_t* indx);
 const uintptr_t* dc_array_get_raw            (const dc_array_t*);
@@ -468,6 +488,7 @@ int             dc_chat_get_archived         (const dc_chat_t*);
 int             dc_chat_is_unpromoted        (const dc_chat_t*);
 int             dc_chat_is_self_talk         (const dc_chat_t*);
 int             dc_chat_is_verified          (const dc_chat_t*);
+int             dc_chat_is_sending_locations (const dc_chat_t*);
 
 
 /**
@@ -486,6 +507,7 @@ int             dc_chat_is_verified          (const dc_chat_t*);
 #define         DC_STATE_IN_FRESH            10
 #define         DC_STATE_IN_NOTICED          13
 #define         DC_STATE_IN_SEEN             16
+#define         DC_STATE_OUT_PREPARING       18
 #define         DC_STATE_OUT_DRAFT           19
 #define         DC_STATE_OUT_PENDING         20
 #define         DC_STATE_OUT_FAILED          24
@@ -520,6 +542,7 @@ int             dc_msg_get_showpadlock        (const dc_msg_t*);
 dc_lot_t*       dc_msg_get_summary            (const dc_msg_t*, const dc_chat_t*);
 char*           dc_msg_get_summarytext        (const dc_msg_t*, int approx_characters);
 int             dc_msg_has_deviating_timestamp(const dc_msg_t*);
+int             dc_msg_has_location           (const dc_msg_t*);
 int             dc_msg_is_sent                (const dc_msg_t*);
 int             dc_msg_is_starred             (const dc_msg_t*);
 int             dc_msg_is_forwarded           (const dc_msg_t*);
@@ -531,6 +554,7 @@ void            dc_msg_set_text               (dc_msg_t*, const char* text);
 void            dc_msg_set_file               (dc_msg_t*, const char* file, const char* filemime);
 void            dc_msg_set_dimension          (dc_msg_t*, int width, int height);
 void            dc_msg_set_duration           (dc_msg_t*, int duration);
+void            dc_msg_set_location           (dc_msg_t*, double latitude, double longitude);
 void            dc_msg_latefiling_mediasize   (dc_msg_t*, int width, int height, int duration);
 
 
@@ -697,51 +721,60 @@ time_t          dc_lot_get_timestamp     (const dc_lot_t*);
  * @{
  */
 
+
 /**
- * Force NORMAL authentification, this is the default.
- * If this, or any other flag is set, automatic configuration is skipped.
+ * Force OAuth2 authorization. This flag does not skip automatic configuration.
+ * Before calling dc_configure() with DC_LP_AUTH_OAUTH2 set,
+ * the user has to confirm access at the URL returned by dc_get_oauth2_url().
+ */
+#define DC_LP_AUTH_OAUTH2                0x2
+
+
+/**
+ * Force NORMAL authorization, this is the default.
+ * If this flag is set, automatic configuration is skipped.
  */
 #define DC_LP_AUTH_NORMAL                0x4
 
 
 /**
  * Connect to IMAP via STARTTLS.
- * If this, or any other flag is set, automatic configuration is skipped.
+ * If this flag is set, automatic configuration is skipped.
  */
 #define DC_LP_IMAP_SOCKET_STARTTLS     0x100
 
 
 /**
  * Connect to IMAP via SSL.
- * If this, or any other flag is set, automatic configuration is skipped.
+ * If this flag is set, automatic configuration is skipped.
  */
 #define DC_LP_IMAP_SOCKET_SSL          0x200
 
 
 /**
  * Connect to IMAP unencrypted, this should not be used.
- * If this, or any other flag is set, automatic configuration is skipped.
+ * If this flag is set, automatic configuration is skipped.
  */
 #define DC_LP_IMAP_SOCKET_PLAIN        0x400
 
 
 /**
  * Connect to SMTP via STARTTLS.
- * If this, or any other flag is set, automatic configuration is skipped.
+ * If this flag is set, automatic configuration is skipped.
  */
 #define DC_LP_SMTP_SOCKET_STARTTLS   0x10000
 
 
 /**
  * Connect to SMTP via SSL.
- * If this, or any other flag is set, automatic configuration is skipped.
+ * If this flag is set, automatic configuration is skipped.
  */
 #define DC_LP_SMTP_SOCKET_SSL        0x20000
 
 
 /**
  * Connect to SMTP unencrypted, this should not be used.
- * If this, or any other flag is set, automatic configuration is skipped.
+ * If this flag is set, automatic configuration is skipped.
  */
 #define DC_LP_SMTP_SOCKET_PLAIN      0x40000 ///<
 
@@ -749,8 +782,7 @@ time_t          dc_lot_get_timestamp     (const dc_lot_t*);
  * @}
  */
 
-#define DC_LP_AUTH_XOAUTH2               0x2
-#define DC_LP_AUTH_FLAGS        (DC_LP_AUTH_XOAUTH2|DC_LP_AUTH_NORMAL) // if none of these flags are set, the default is choosen
+#define DC_LP_AUTH_FLAGS        (DC_LP_AUTH_OAUTH2|DC_LP_AUTH_NORMAL) // if none of these flags are set, the default is choosen
 #define DC_LP_IMAP_SOCKET_FLAGS (DC_LP_IMAP_SOCKET_STARTTLS|DC_LP_IMAP_SOCKET_SSL|DC_LP_IMAP_SOCKET_PLAIN) // if none of these flags are set, the default is choosen
 #define DC_LP_SMTP_SOCKET_FLAGS (DC_LP_SMTP_SOCKET_STARTTLS|DC_LP_SMTP_SOCKET_SSL|DC_LP_SMTP_SOCKET_PLAIN) // if none of these flags are set, the default is choosen
 
@@ -974,6 +1006,19 @@ time_t          dc_lot_get_timestamp     (const dc_lot_t*);
 #define DC_EVENT_CONTACTS_CHANGED         2030
 
 
+
+/**
+ * Location of one or more contact has changed.
+ *
+ * @param data1 (int) contact_id of the contact for which the location has changed.
+ *     If the locations of several contacts have been changed,
+ *     eg. after calling dc_delete_all_locations(), this parameter is set to 0.
+ * @param data2 0
+ * @return 0
+ */
+#define DC_EVENT_LOCATION_CHANGED         2035
+
+
 /**
  * Inform about the configuration progress started by dc_configure().
  *
@@ -1062,20 +1107,40 @@ time_t          dc_lot_get_timestamp     (const dc_lot_t*);
 
 
 /**
- * Request a HTTP-file or HTTPS-file from the frontend.
+ * Request a HTTP-file or HTTPS-file from the frontend using HTTP-GET.
  *
  * @param data1 (const char*) Null-terminated UTF-8 string containing the URL.
  *     The string starts with https:// or http://.
  *     Must not be free()'d or modified and is valid only until the callback returns.
  * @param data2 0
  * @return (const char*) The content of the requested file as a null-terminated UTF-8 string;
- *     Response headers, encodings etc. must be stripped, only the raw file, which may be binary, should be returned.
+ *     Response headers, encodings etc. must be stripped.
+ *     Only the raw file should be returned.
  *     CAVE: The string will be free()'d by the core,
  *     so make sure it is allocated using malloc() or a compatible function.
  *     If you cannot provide the content, just return 0 or an empty string.
  */
 #define DC_EVENT_HTTP_GET                 2100
 
+
+/**
+ * Request a HTTP-file or HTTPS-file from the frontend using HTTP-POST.
+ *
+ * @param data1 (const char*) Null-terminated UTF-8 string containing the URL.
+ *     The string starts with https:// or http://.
+ *     Must not be free()'d or modified and is valid only until the callback returns.
+ *     Parameter to POST are added to the url after `?`.
+ * @param data2 0
+ * @return (const char*) The content of the requested file as a null-terminated UTF-8 string;
+ *     Response headers, encodings etc. must be stripped.
+ *     Only the raw file should be returned.
+ *     CAVE: The string will be free()'d by the core,
+ *     so make sure it is allocated using malloc() or a compatible function.
+ *     If you cannot provide the content, just return 0 or an empty string.
+ */
+#define DC_EVENT_HTTP_POST                2110
+
+
 /**
  * @}
  */
@@ -1091,6 +1156,14 @@ time_t          dc_lot_get_timestamp     (const dc_lot_t*);
 #define DC_EVENT_RETURNS_STRING(e)   ((e)==DC_EVENT_GET_STRING || (e)==DC_EVENT_HTTP_GET)
 
 
+/*
+ * Values for dc_get|set_config("show_emails")
+ */
+#define DC_SHOW_EMAILS_OFF               0
+#define DC_SHOW_EMAILS_ACCEPTED_CONTACTS 1
+#define DC_SHOW_EMAILS_ALL               2
+
+
 /*
  * TODO: Strings need some doumentation about used placeholders.
  *
@@ -1142,7 +1215,10 @@ time_t          dc_lot_get_timestamp     (const dc_lot_t*);
 #define DC_STR_SERVER_RESPONSE            61
 #define DC_STR_MSGACTIONBYUSER            62
 #define DC_STR_MSGACTIONBYME              63
-#define DC_STR_COUNT                      64
+#define DC_STR_MSGLOCATIONENABLED         64
+#define DC_STR_MSGLOCATIONDISABLED        65
+#define DC_STR_LOCATION                   66
+#define DC_STR_COUNT                      66
 
 /*
  * @}
@@ -1153,4 +1229,3 @@ time_t          dc_lot_get_timestamp     (const dc_lot_t*);
 }
 #endif
 #endif // __DELTACHAT_H__
-
